@@ -1,13 +1,13 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #########################################################
-## Thai Language Toolkit : version  1.0.9
+## Thai Language Toolkit : version  1.2.8
 ## Chulalongkorn University
 ## written by Wirote Aroonmanakun
 ## Implemented :
-##      loadTNC(), load3gram(Filename), unigram(w1), bigram(w1,w2), trigram(w1,w2,w3)
+##      TNC_load(), TNC3g_load(), trigram_load(Filename), unigram(w1), bigram(w1,w2), trigram(w1,w2,w3)
 ##      collocates(w,SPAN,STAT,DIR,LIMIT, MINFQ) = [wa,wb,wc]
-##      similarlity(w1,w2) 
+##      w2v_exist(w), similar_words(w), similarlity(w1,w2), outofgroup(WORDLST), analogy(w1,w2,w3) 
 #########################################################
 
 import re
@@ -16,9 +16,50 @@ import math
 from collections import defaultdict
 from operator import itemgetter
 import gensim
+import pickle
+from sklearn.decomposition import PCA
+try:
+    import matplotlib
+    from matplotlib import pyplot
+except ImportError:
+    mplAvailable = False
+else:
+    mplAvailable = True
+import numpy
+
 #import gensim, logging
 #logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
 
+def TNC3g_load():
+    global TriCount
+    global BiCount
+    global BiCount2
+    global UniCount
+    global TotalWord
+
+    TriCount = defaultdict(int)
+    BiCount = defaultdict(int)
+    UniCount = defaultdict(int)
+    BiCount2 = defaultdict(int)
+    TotalWord = 0
+
+    path = os.path.abspath(__file__)
+    ATA_PATH = os.path.dirname(path)
+    try: 
+        fileObject = open(ATA_PATH + '/TNC3g','rb')
+    except IOError:
+        fileObject = open('TNC3g','rb') 
+    TriCount = pickle.load(fileObject)
+
+    for (w1,w2,w3) in TriCount:
+        freq = int(TriCount[(w1,w2,w3)])
+        BiCount[(w1,w2)] += freq
+        UniCount[w1] += freq
+        BiCount2[(w1,w3)] += freq
+        TotalWord += freq
+    return(1)
+
+
 
 def TNC_load():
     global TriCount
@@ -94,6 +135,11 @@ def bigram(w1,w2):
     global BiCount
     global TotalWord
     
+    try:
+      BiCount
+    except NameError:
+      TNC_load()
+
     if (w1,w2) in BiCount:
         return(float(BiCount[(w1,w2)] * 1000000 / TotalWord))
     else:
@@ -264,10 +310,10 @@ def w2v_load():
 
 def w2v_exist(w):
     global w2v_model
-#    try:
-#      w2v_model
-#    except NameError:
-#      w2v_load()
+    try:
+      w2v_model
+    except NameError:
+      w2v_load()
     if w in list(w2v_model.wv.vocab):
         return(True)
     else:
@@ -285,8 +331,21 @@ def similarity(w1,w2):
     vocabs = list(w2v_model.wv.vocab)
     if w1 in vocabs and w2 in vocabs:
         degree = w2v_model.similarity(w1,w2)
+    else:
+        degree = 0.    
     return(degree)
 
+def cosine_similarity(w1,w2):
+    global w2v_model
+    degree = ''
+    vocabs = list(w2v_model.wv.vocab)
+    if w1 in vocabs and w2 in vocabs:
+        degree = cosine_similarity = numpy.dot(w2v_model[w1], w2v_model[w2])/(numpy.linalg.norm(w2v_model[w1])* numpy.linalg.norm(w2v_model[w2]))
+    else:
+        degree = 0.    
+    return(degree)
+
+
 def similar_words(w1,n=10,cutoff=0.,score="n"):
     global w2v_model                
     if w1 in list(w2v_model.wv.vocab):
@@ -316,6 +375,80 @@ def outofgroup(wrdlst):
     out = w2v_model.doesnt_match(wrdlst1)
     return(out)
 
+def w2v_diffplot(ww,wx,wy):
+    global w2v_model
+    try:
+      w2v_model
+    except NameError:
+      w2v_load()
+    font = {'family' : 'TH Sarabun New',
+        'weight' : 'bold',
+        'size'   : 14}
+    matplotlib.rc('font', **font)
+    xs = list(range(1,101))
+    ys = w2v_model.wv[ww] - w2v_model.wv[wx]
+    pyplot.plot(xs,ys,label=ww+'-'+wx)
+    ys = w2v_model.wv[ww] - w2v_model.wv[wy]
+    pyplot.plot(xs,ys,label=ww+'-'+wy)
+    pyplot.legend()
+    pyplot.show()
+    return(1)
+
+def w2v_dimplot(wrdlst):
+    global w2v_model
+    wrdlst1=[]
+    try:
+      w2v_model
+    except NameError:
+      w2v_load()
+    vocabs = list(w2v_model.wv.vocab)
+    for w in wrdlst:
+        if w in vocabs:
+            wrdlst1.append(w)
+    font = {'family' : 'TH Sarabun New',
+        'weight' : 'bold',
+        'size'   : 14}
+    matplotlib.rc('font', **font)
+    xs = list(range(1,101))
+    for i, word in enumerate(wrdlst1):
+        ys = w2v_model.wv[word]
+        pyplot.plot(xs,ys,label=word)
+    pyplot.legend()
+    pyplot.show()
+    return(1)
+    
+
+def w2v_plot(wrdlst):
+    global w2v_model
+    wrdlst1=[]
+    try:
+      w2v_model
+    except NameError:
+      w2v_load()
+    vocabs = list(w2v_model.wv.vocab)
+    for w in wrdlst:
+        if w in vocabs:
+            wrdlst1.append(w)
+    # fit a 2d PCA model to the vectors
+    font = {'family' : 'TH Sarabun New',
+        'weight' : 'bold',
+        'size'   : 14}
+    matplotlib.rc('font', **font)
+
+    X = w2v_model[wrdlst1]
+    pca = PCA(n_components=2)
+    result = pca.fit_transform(X)
+    # create a scatter plot of the projection
+    ax = pyplot.axes()
+    pyplot.scatter(result[:, 0], result[:, 1])
+    for i, word in enumerate(wrdlst1):
+        pyplot.annotate(word, xy=(result[i, 0], result[i, 1]))
+        ax.arrow(0, 0, result[i, 0], result[i, 1], head_width=0.05, head_length=0.1)
+    pyplot.show()
+    return(1)
+
+    
+
 def analogy(w1,w2,w3, n=1):   #king - man + woman = queen    
     global w2v_model
     try:
@@ -328,6 +461,42 @@ def analogy(w1,w2,w3, n=1):   #king - man + woman = queen
     else:
         return([])
 
+#### codes adapted from http://chrisculy.net/lx/wordvectors/wvecs_visualization.html
+def w2v_compare_color(wds):
+    global w2v_model
+    wdsr = wds[:]
+    wdsr.reverse()
+
+    font = {'family' : 'TH Sarabun New',
+        'weight' : 'bold',
+        'size'   : 14}
+    matplotlib.rc('font', **font)
+    
+#    display(HTML('<b>Word vectors for: %s</b>' % ', '.join(wdsr)))
+    
+    vs = [w2v_model.wv[wd] for wd in wds]
+    dim = len(vs[0])
+    
+    fig = pyplot.figure(num=None, figsize=(12, 2), dpi=80, facecolor='w', edgecolor='k')
+    ax = fig.add_subplot(111)
+    ax.set_facecolor('gray')
+    
+    for i,v in enumerate(vs):
+        ax.scatter(range(dim),[i]*dim, c=vs[i], cmap='Spectral', s=16)
+    
+    #plt.xticks(range(n), [i+1 for i in range(n)])
+    pyplot.xlabel('Dimension')
+    pyplot.yticks(range(len(wds)), wds)
+    
+    pyplot.show()        
+
 ############ END OF GENERAL MODULES ##########################################################################
 
 
+## testing area
+#w2v_load()
+#w2v_plot("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
+#w2v_compare_color("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
+#w2v_plot(['เก็บรักษา', 'จัดเตรียม','เอา', 'รวบรวม', 'ซื้อ', 'สะสม'])
+#w2v_diffplot("เล็กน้อย","เล็ก", "น้อย")
+#w2v_plot(["แทรกซ้อน","แทรก", "ซ้อน"])
\ No newline at end of file
